@@ -41,10 +41,6 @@ class Patch(AnyEntity):
     def dc_title(self):
         return self.patch_name
 
-    @property
-    def patch_name(self):
-        return self.revisions[-1].file.path
-
     @property
     def repository(self):
         return self.patch_repository[0]
