@@ -49,6 +49,9 @@ class Patch(WorkflowableEntityType):
     originator = String(
         maxsize=256, indexed=True,
         description=_('author of the revision which has introduced this patch'))
+    patch_name = String(
+        required=True,
+        description=_('name of the patch in the repository'))
 
 
 class patch_repository(RelationDefinition):
@@ -116,8 +119,8 @@ class patch_reviewer(RelationDefinition):
         }
     subject = 'Patch'
     object = 'CWUser'
-    constraints = [RQLConstraint('EXISTS(U in_group G, G name "reviewers") '
-                                 'OR EXISTS(X patch_repository R, R repository_reviewer U)')]
+    constraints = [RQLConstraint('EXISTS(O in_group G, G name "reviewers") '
+                                 'OR EXISTS(S patch_repository R, R repository_reviewer O)')]
 
 class patch_depends_on(RelationDefinition):
     subject = 'Patch'
