@@ -23,7 +23,10 @@ class PatchReviewWorkflowTC(CubicWebTC):
 
     def setup_database(self):
         req = self.request()
-        self.reviewer = self.create_user(req, 'reviewer', ('users', 'reviewers'))
+        self.reviewer1 = self.create_user(req, 'reviewer1', ('users', 'reviewers'),
+                                          email=u'reviewer1@cwo')
+        self.reviewer2 = self.create_user(req, 'reviewer2', ('users', 'reviewers'),
+                                          email=u'reviewer2@cwo')
         self.committer = self.create_user(req, 'committer', ('users',))
         session = self.session
         session.set_pool()
@@ -31,35 +34,53 @@ class PatchReviewWorkflowTC(CubicWebTC):
             with hooks_control(session, session.HOOKS_DENY_ALL, 'metadata'):
                 vcrepo = session.create_entity(
                     'Repository', repository_committer=self.committer)
-                patch = session.create_entity(
-                    'Patch', patch_repository=vcrepo)
+                self.patch1eid = session.create_entity(
+                    'Patch', patch_repository=vcrepo, originator=u'bob <reviewer1@cwo>').eid
+                self.patch2eid = session.create_entity(
+                    'Patch', patch_repository=vcrepo, originator=u'bob <reviewer1@cwo>').eid
                 session.commit(reset_pool=False)
 
     def test_wf(self):
         req = self.request()
-        patch = req.execute('Patch X').get_entity(0, 0)
+        patch = req.execute('Any X WHERE X eid %(x)s', {'x': self.patch1eid}).get_entity(0, 0)
         self.failIf(patch.patch_reviewer)
+        self.failUnless(self.reviewer1.eid in [u.eid for u in patch.created_by])
+        self.failUnless(self.reviewer1.eid in [u.eid for u in patch.owned_by])
         patch.cw_adapt_to('IWorkflowable').fire_transition('ask review')
         self.commit()
         patch.cw_clear_all_caches()
-        self.assertEqual(patch.patch_reviewer[0].eid, self.reviewer.eid)
-        with self.login('reviewer'):
+        self.assertEqual(patch.patch_reviewer[0].eid, self.reviewer2.eid)
+        with self.login('reviewer1'):
             req = self.request()
-            rset = req.execute('Any X WHERE X is Patch')
+            rset = req.execute('Any X WHERE X eid %(x)s', {'x': self.patch1eid})
             self.assertListEqual(sorted(x[0] for x in self.action_submenu(req, rset, 'workflow')),
-                                 ['accept', 'fold', 'refuse', 'view history', 'view workflow'])
+                                 ['accept', 'ask rework', 'fold', 'view history', 'view workflow'])
+            # reviewer1 submit its second patch for review: while reviewer2
+            # already has a patch for review, it should be picked anyway as a
+            # patch creator can't be its reviewer
+            patch = req.execute('Any X WHERE X eid %(x)s', {'x': self.patch2eid}).get_entity(0, 0)
+            self.failIf(patch.patch_reviewer)
+            patch.cw_adapt_to('IWorkflowable').fire_transition('ask review')
+            self.commit()
+            patch.cw_clear_all_caches()
+            self.assertEqual(patch.patch_reviewer[0].eid, self.reviewer2.eid)
+        with self.login('reviewer2'):
+            req = self.request()
+            rset = req.execute('Any X WHERE X eid %(x)s', {'x': self.patch1eid})
+            self.assertListEqual(sorted(x[0] for x in self.action_submenu(req, rset, 'workflow')),
+                                 ['accept', 'ask rework', 'fold', 'view history', 'view workflow'])
             patch = rset.get_entity(0, 0)
             patch.cw_adapt_to('IWorkflowable').fire_transition('accept')
             self.commit()
             patch.cw_clear_all_caches()
             self.assertListEqual(sorted(x[0] for x in self.action_submenu(req, rset, 'workflow')),
-                                 ['fold', 'refuse', 'view history', 'view workflow'])
+                                 ['ask rework', 'fold', 'view history', 'view workflow'])
         with self.login('committer'):
             req = self.request()
-            patch = req.execute('Patch X').get_entity(0, 0)
-            rset = req.execute('Any X WHERE X is Patch')
+            rset = req.execute('Any X WHERE X eid %(x)s', {'x': self.patch1eid})
             self.assertListEqual(sorted(x[0] for x in self.action_submenu(req, rset, 'workflow')),
-                                 ['apply', 'fold', 'refuse', 'reject', 'view history', 'view workflow'])
+                                 ['apply', 'ask rework', 'fold', 'reject', 'view history', 'view workflow'])
+            patch = rset.get_entity(0, 0)
             patch.cw_adapt_to('IWorkflowable').fire_transition('apply')
             self.commit()
             patch.cw_clear_all_caches()
@@ -67,6 +88,7 @@ class PatchReviewWorkflowTC(CubicWebTC):
                                  ['view history'])
 
 
+
 if __name__ == '__main__':
     from logilab.common.testlib import unittest_main
     unittest_main()
