@@ -84,7 +84,8 @@ class PatchReviewer(view.EntityView):
     __regid__ = 'vcreview.patch.reviewers'
     __select__ = is_instance('Patch')
     def entity_call(self, entity, **kwargs):
-        return self._cw.view('csv', entity.related('patch_reviewer'), 'null')
+        self._cw.view('csv', entity.related('patch_reviewer'), 'null',
+                      w=self.w)
 
 
 class WorkListLink(component.CtxComponent):
